@@ -1,4 +1,5 @@
 ## Copyright 2014-2016 Oliver Heimlich
+## Copyright 2017 Joel Dahne
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -17,10 +18,10 @@
 ## @documentencoding UTF-8
 ## @defop Method {@@infsup} gt (@var{A}, @var{B})
 ## @defopx Operator {@@infsup} {@var{A} > @var{B}}
-## 
+##
 ## Compare intervals @var{A} and @var{B} for strict greater.
 ##
-## Evaluated on interval matrices, this functions is applied element-wise.
+## Evaluated on interval arrays, this functions is applied element-wise.
 ##
 ## @seealso{@@infsup/eq, @@infsup/lt, @@infsup/ge}
 ## @end defop
@@ -31,12 +32,12 @@
 
 function result = gt (a, b)
 
-if (nargin ~= 2)
+  if (nargin ~= 2)
     print_usage ();
     return
-endif
+  endif
 
-result = lt (b, a);
+  result = lt (b, a);
 
 endfunction
 
