@@ -1,5 +1,8 @@
 /*
   Copyright 2015-2016 Oliver Heimlich
+  Copyright 2017 Joel Dahne
+  Copyright 2009-2017 Jaroslav Hajek
+  Copyright 2009 VZLU Prague
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -17,7 +20,7 @@
 
 #include <octave/oct.h>
 #include <mpfr.h>
-#include "mpfr_commons.cc"
+#include "mpfr_commons.h"
 
 typedef int (*mpfr_unary_fun)
             (mpfr_t rop, const mpfr_t op,
@@ -32,9 +35,9 @@ typedef int (*mpfr_ternary_fun)
                          const mpfr_t op3,
                          mpfr_rnd_t rnd);
 
-// Evaluate an unary MPFR function on a binary64 matrix
+// Evaluate an unary MPFR function on a binary64 array
 void evaluate (
-  Matrix &arg1,           // Operand 1 and result
+  NDArray &arg1,          // Operand 1 and result
   const mpfr_rnd_t rnd,   // Rounding direction
   const mpfr_unary_fun f) // The MPFR function to evaluate (element-wise)
 {
@@ -60,10 +63,10 @@ void evaluate (
   mpfr_set_emin (old_emin);
 }
 
-// Evaluate a binary MPFR function on two binary64 matrices
+// Evaluate a binary MPFR function on two binary64 arrays
 void evaluate (
-  Matrix &arg1,            // Operand 1 and result
-  const Matrix &arg2,      // Operand 2
+  NDArray &arg1,           // Operand 1 and result
+  const NDArray &arg2,     // Operand 2
   const mpfr_rnd_t rnd,    // Rounding direction
   const mpfr_binary_fun f) // The MPFR function to evaluate (element-wise)
 {
@@ -73,49 +76,96 @@ void evaluate (
   mpfr_exp_t old_emin = mpfr_get_emin ();
   mpfr_set_emin (BINARY64_EMIN);
 
-  // arg1 shall contain the result and must be resized
-  if (arg1.rows () == 1 && arg2.rows () != 1)
-    arg1 = arg1.index (idx_vector (ColumnVector (arg2.rows (), 1.0)),
-                       idx_vector::colon);
-  if (arg1.columns () == 1 && arg2.columns () != 1)
-    arg1 = arg1.index (idx_vector::colon,
-                       idx_vector (RowVector (arg2.columns (), 1.0)));
+  int dimensions = std::max (arg1.ndims (), arg2.ndims ());
+  dim_vector arg1_dims = arg1.dims().redim (dimensions);
+  dim_vector arg2_dims = arg2.dims().redim (dimensions);
+  dim_vector arg1_cdims = arg1_dims.cumulative ();
+  dim_vector arg2_cdims = arg2_dims.cumulative ();
 
-  const octave_idx_type n = arg1.rows ();
-  const octave_idx_type m = arg1.columns ();
-  const bool broadcast_r = n != 1 && arg2.rows () == 1;
-  const bool broadcast_c = m != 1 && arg2.columns () == 1;
+  // Create result array of right size
+  dim_vector result_dims;
+  result_dims.resize (dimensions);
 
-  for (octave_idx_type i = 0; i < n; i ++)
-    for (octave_idx_type j = 0; j < m; j ++)
+  for (int i = 0; i < dimensions; i ++)
+    {
+      if (arg1_dims(i) != 1)
+        result_dims(i) = arg1_dims(i);
+      else
+        result_dims(i) = arg2_dims(i);
+    }
+
+  NDArray result (result_dims);
+
+  // Find the first dimension that needs broadcasting
+  octave_idx_type start;
+  octave_idx_type step = 1;
+  for (start = 0; start < dimensions; start ++)
+  {
+    if (arg1_dims(start) != arg2_dims(start))
+      break;
+    step = arg1_cdims (start);
+  }
+
+  // Fix broadcasting along all singleton dimensions
+  // Does not work for broadcasting along the first dimension
+  for (int i = std::max (start, static_cast <octave_idx_type> (1)); i < dimensions; i ++)
+  {
+    if (arg1_dims(i) == 1)
+      arg1_cdims(i-1) = 0;
+    if (arg2_dims(i) == 1)
+      arg2_cdims(i-1) = 0;
+  }
+
+  // Perform the operation
+  octave_idx_type arg1_idx;
+  octave_idx_type arg2_idx;
+  OCTAVE_LOCAL_BUFFER_INIT (octave_idx_type, idx_base, dimensions, 0);
+
+  octave_idx_type n = result.numel ();
+
+  for (octave_idx_type iter = 0; iter < n; iter += step)
+  {
+    // Take broadcasting into account
+    arg1_idx = arg1_cdims.cum_compute_index (idx_base);
+    arg2_idx = arg2_cdims.cum_compute_index (idx_base);
+    // Broadcasting along the first dimension needs to be handled
+    // separately
+    if (start == 0)
       {
-        mpfr_set_d (mp1, arg1.elem (i, j), MPFR_RNDZ);
-        mpfr_set_d (mp2,
-                    (broadcast_r)
-                      ? ((broadcast_c) ? arg2.elem (0, 0)
-                                       : arg2.elem (0, j))
-                      : ((broadcast_c) ? arg2.elem (i, 0)
-                                       : arg2.elem (i, j))
-                    , MPFR_RNDZ);
+        if (arg1_dims(0) == 1)
+          arg1_idx -= idx_base[0];
+        else
+          arg2_idx -= idx_base[0];
+      }
+
+    for (octave_idx_type i = 0; i < step; i ++)
+      {
+        mpfr_set_d (mp1, arg1.elem (arg1_idx + i), MPFR_RNDZ);
+        mpfr_set_d (mp2, arg2.elem (arg2_idx + i), MPFR_RNDZ);
         int rnd_error = (*f) (mp1, mp1, mp2, rnd);
         if (rnd == MPFR_RNDN)
           {
             // Prevent double-rounding errors
             mpfr_subnormalize (mp1, rnd_error, rnd);
           }
-        arg1.elem (i, j) = mpfr_get_d (mp1, rnd);
+        result.elem (iter + i) = mpfr_get_d (mp1, rnd);
       }
 
+    result_dims.increment_index (idx_base + start, start);
+  }
+
+  arg1 = result;
+
   mpfr_clear (mp1);
   mpfr_clear (mp2);
   mpfr_set_emin (old_emin);
 }
 
-// Evaluate a ternary MPFR function on three binary64 matrices
+// Evaluate a ternary MPFR function on three binary64 arrays
 void evaluate (
-  Matrix &arg1,             // Operand 1 and result
-  const Matrix &arg2,       // Operand 2
-  const Matrix &arg3,       // Operand 3
+  NDArray &arg1,            // Operand 1 and result
+  const NDArray &arg2,      // Operand 2
+  const NDArray &arg3,      // Operand 3
   const mpfr_rnd_t rnd,     // Rounding direction
   const mpfr_ternary_fun f) // The MPFR function to evaluate (element-wise)
 {
@@ -126,38 +176,16 @@ void evaluate (
   mpfr_exp_t old_emin = mpfr_get_emin ();
   mpfr_set_emin (BINARY64_EMIN);
 
-  bool scalar1 = arg1.numel () == 1;
-  bool scalar2 = arg2.numel () == 1;
-  bool scalar3 = arg3.numel () == 1;
+  // Note that no broadcasting is performed here, this is because
+  // currently no ternary functions needs broadcasting
 
-  if (scalar1)
-    {
-      // arg1 shall contain the result and must possibly be resized
-      if (!scalar2)
-        {
-          arg1 = Matrix (arg2.dims (), arg1.elem (0));
-          scalar1 = false;
-        }
-      else if (!scalar3)
-        {
-          arg1 = Matrix (arg3.dims (), arg1.elem (0));
-          scalar1 = false;
-        }
-    }
+  const octave_idx_type n = arg1.numel ();
 
-  const octave_idx_type n = std::max (std::max (arg1.numel (), arg2.numel ()),
-                                   arg3.numel ());
   for (octave_idx_type i = 0; i < n; i ++)
     {
       mpfr_set_d (mp1, arg1.elem (i), MPFR_RNDZ);
-      mpfr_set_d (mp2,
-                  (scalar2) ? arg2.elem (0) // broadcast
-                            : arg2.elem (i),
-                  MPFR_RNDZ);
-      mpfr_set_d (mp3,
-                  (scalar3) ? arg3.elem (0) // broadcast
-                            : arg3.elem (i),
-                  MPFR_RNDZ);
+      mpfr_set_d (mp2, arg2.elem (i), MPFR_RNDZ);
+      mpfr_set_d (mp3, arg3.elem (i), MPFR_RNDZ);
       int rnd_error = (*f) (mp1, mp1, mp2, mp3, rnd);
       if (rnd == MPFR_RNDN)
         {
@@ -175,18 +203,21 @@ void evaluate (
 
 // Evaluate nthroot
 void nthroot (
-  Matrix &arg1,             // Operand 1 and result
-  const uint64_t arg2, // Operand 2
+  NDArray &arg1,       // Operand 1 and result
+  const uint64NDArray arg2, // Operand 2
   const mpfr_rnd_t rnd)
 {
   mpfr_t mp;
   mpfr_init2 (mp, BINARY64_PRECISION);
 
+  // Note that no broadcasting is performed here, this is because
+  // that nthroot performs the broadcasting in the m-file
+
   const octave_idx_type n = arg1.numel ();
   for (octave_idx_type i = 0; i < n; i ++)
     {
       mpfr_set_d (mp, arg1.elem (i), MPFR_RNDZ);
-      mpfr_root (mp, mp, arg2, rnd);
+      mpfr_root (mp, mp, static_cast <uint64_t> (arg2.elem(i)), rnd);
       arg1.elem (i) = mpfr_get_d (mp, rnd);
     }
 
@@ -195,7 +226,7 @@ void nthroot (
 
 // Evaluate factorial
 void factorial (
-  Matrix &arg1, // Operand 1 and result
+  NDArray &arg1, // Operand 1 and result
   const mpfr_rnd_t rnd)
 {
   mpfr_t mp;
@@ -285,6 +316,7 @@ DEFUN_DLD (mpfr_function_d, args, nargout,
   "@defunx mpfr_function_d ('psi', @var{R}, @var{X})\n"
   "@defunx mpfr_function_d ('rdivide', @var{R}, @var{X}, @var{Y})\n"
   "@defunx mpfr_function_d ('realsqrt', @var{R}, @var{X})\n"
+  "@defunx mpfr_function_d ('rem', @var{R}, @var{X}, @var{Y})\n"
   "@defunx mpfr_function_d ('rsqrt', @var{R}, @var{X})\n"
   "@defunx mpfr_function_d ('sec', @var{R}, @var{X})\n"
   "@defunx mpfr_function_d ('sech', @var{R}, @var{X})\n"
@@ -303,8 +335,9 @@ DEFUN_DLD (mpfr_function_d, args, nargout,
   "@option{-inf}: towards negative infinity).  "
   "Parameters 3 and (possibly) 4 and 5 are operands to the function."
   "\n\n"
-  "Evaluated on matrices, the function will be applied element-wise.  Scalar "
-  "operands do broadcast for functions with more than one operand."
+  "Evaluated on arrays, the function will be applied element-wise.  "
+  "For binary functions broadcasting is performed where needed.  "
+  "For ternary functions no broadcasting is performed.  "
   "\n\n"
   "The result is guaranteed to be correctly rounded.  That is, the function "
   "is evaluated with (virtually) infinite precision and the exact result is "
@@ -333,26 +366,30 @@ DEFUN_DLD (mpfr_function_d, args, nargout,
   // Read parameters
   const std::string function = args (0).string_value ();
   const mpfr_rnd_t  rnd      = parse_rounding_mode (args (1).scalar_value ());
-  Matrix            arg1     = args (2).matrix_value ();
-  Matrix            arg2;
-  Matrix            arg3;
+  NDArray           arg1     = args (2).array_value ();
+  NDArray           arg2;
+  NDArray           arg3;
   if (nargin >= 4)
     {
-      arg2                   = args (3).matrix_value ();
-      if (arg1.rows () != 1 && arg2.rows () != 1 &&
-          arg1.rows () != arg2.rows ())
-        error ("mpfr_function_d: Matrix dimensions must agree!");
-      if (arg1.columns () != 1 && arg2.columns () != 1 &&
-          arg1.columns () != arg2.columns ())
-        error ("mpfr_function_d: Matrix dimensions must agree!");
+      arg2                   = args (3).array_value ();
+      // Check if broadcasting can be performed
+      int dimensions = std::max (arg1.ndims (), arg2.ndims ());
+      dim_vector arg1_dims = arg1.dims().redim (dimensions);
+      dim_vector arg2_dims = arg2.dims().redim (dimensions);
+      for (int dim = 0; dim < dimensions; dim ++)
+        {
+          if (arg1_dims (dim) != 1 && arg2_dims (dim) != 1 &&
+              arg1_dims (dim) != arg2_dims (dim))
+            error ("mpfr_function_d: Array dimensions must agree!");
+        }
     }
   if (nargin >= 5)
     {
-      arg3                   = args (4).matrix_value ();
-      if (arg3.numel () != 1 && (
-          (arg1.numel () != 1 && arg1.numel () != arg3.numel ()) ||
-          (arg2.numel () != 1 && arg2.numel () != arg3.numel ())))
-        error ("mpfr_function_d: Matrix dimensions must agree!");
+      arg3                   = args (4).array_value ();
+      // We never perform broadcasting for ternary functions so all
+      // dimensions must be equal
+      if (arg1.dims () != arg2.dims () || arg2.dims () != arg3.dims ())
+        error ("mpfr_function_d: Array dimensions must agree!");
     }
   if (error_state)
     return octave_value_list ();
@@ -453,7 +490,7 @@ DEFUN_DLD (mpfr_function_d, args, nargout,
           evaluate (arg1, arg2, rnd, &mpfr_sub);
         else if (function == "nthroot")
           {
-            const uint64_t n = args (3).uint64_array_value ().elem (0);
+            uint64NDArray n = args (3).uint64_array_value ();
             if (error_state)
               return octave_value_list ();
             nthroot (arg1, n, rnd);
@@ -464,6 +501,8 @@ DEFUN_DLD (mpfr_function_d, args, nargout,
           evaluate (arg1, arg2, rnd, &mpfr_pow);
         else if (function == "rdivide")
           evaluate (arg1, arg2, rnd, &mpfr_div);
+        else if (function == "rem")
+          evaluate (arg1, arg2, rnd, &mpfr_fmod);
         else if (function == "times")
           evaluate (arg1, arg2, rnd, &mpfr_mul);
         else
